#include <stdio.h>
#include <stdlib.h>
#include <spritepack.h>

/* Extern pointers for the graphic definitions: */

#include "map.h"
#include "sprites.h"
#include "tiles.h"

/* Allocate space for the stack */

#pragma output STACKPTR=61440

/* Addresses */

#define AD_FREE		60000

/* Game constants */

#define SPEED_FALLING 4
#define SPEED_WALKING 2
#define SPEED_JUMPY 4
#define SPEED_JUMPX 2
#define MAX_NU 16
#define HALF_NU 8
#define COUNTER_SET 25
#define LIFE 127
#define TIME 30
#define ADDLIFE 16

#define WYZ_INTERRUPT 24520
#define WYZ_LOAD 24563
#define WYZ_INTERR 25266
#define WYZ_PLAY_SOUND 24548

#define SCR_CEZ			28727
#define SCR_MARCADOR	30183
#define SCR_FIN			28964
#define PUCRUNCH		33163

char play = 0;

#asm 
defw 0                        /* two free bytes before start of a hook */ 
#endasm 
void musicISR(void) 
{ 
	if (play)
	{
	#asm 
			push af
			push bc
			push de
			push hl
			push ix

			call WYZ_INTERRUPT 

			pop ix
			pop hl
			pop de
			pop bc
			pop af
	#endasm 
	}
} 

/* Import sp_ClipStruct */

extern struct sp_Rect *sp_ClipStruct;

#asm
	LIB SPCClipStruct
._sp_ClipStruct         
	defw SPCClipStruct
#endasm

struct sp_Rect spritesClipValues;
struct sp_Rect *spritesClip;

/* Manage keys and joystick at the same time: */

struct sp_UDK keys;

/* Create memory allocator for sprite routines */

void *my_malloc(uint bytes)
{
   return sp_BlockAlloc(0);
}

void *u_malloc = my_malloc;
void *u_free = sp_FreeBlock;

/* Strings printed during the game */

struct sp_PSS ps;         /* a print string struct used by sp_PrintString */

/* Data types */

typedef struct
{
	uchar w, a;
	char sal, nu, mw, ma;
	char facing, frame;
	uchar *current_frame, *next_frame;
	char falling;
	unsigned char life, keys, bolts;
	unsigned char time_left, time_counter;
} PHANTOMAS; 

PHANTOMAS phantomas;

typedef struct
{
	char type;								// Tipo de enemigo: 0 lin horz, 1 lin vert, 2 rand, 3 chaser | (todo se mide en TILES)
	uchar t1, t2;							// lmites en enemigos lin, x/y en enemigos rand o chaser
	uchar p;								// pos horz/vert en enemigos lin
	char v;									// velocidad.
	char spr, c;							// Nmero de enemigo y su color.
} ENE_MAPA;

ENE_MAPA ene_mapa [75] = {
	// p 0 0
		{0,6,13,0,1,0,1},
		{1,0,4,5,2,0,2},
		{1,2,6,12,2,0,3},
	// p 0 1
		{0,0,4,4,1,0,4},
		{0,5,9,3,2,1,5},
		{0,11,14,4,1,1,6},
	// p 0 2 
		{1,1,5,4,4,2,7},
		{1,2,5,5,2,2,1},
		{1,3,5,6,1,2,2},
	// p 0 3
		{1,0,7,2,4,0,3},
		{1,0,7,12,4,0,3},
		{0,3,11,1,1,1,5},
	// p 0 4
		{0,4,7,1,2,1,6},
		{0,0,5,6,4,2,4},
		{0,7,12,6,2,2,1},

	// p 1 0
		{0,4,6,3,1,1,1},
		{0,8,11,3,2,2,2},
		{0,1,13,6,4,0,3},
	// p 1 1
		{1,1,5,4,2,0,4},
		{1,1,6,9,12,0,5},
		{1,1,7,13,4,2,6},
	// p 1 2
		{1,1,4,3,1,2,7},
		{1,1,4,11,1,2,1},
		{0,4,10,4,2,1,2},
	// p 1 3
		{1,2,5,1,4,2,3},
		{0,2,5,0,1,1,4},
		{1,0,4,12,1,0,5},
	// p 1 4
		{1,1,4,1,1,2,6},
		{0,7,9,3,2,1,7},
		{1,1,5,13,2,0,1},

	// p 2 0
		{1,0,7,6,1,0,2},
		{1,0,7,7,2,1,4},
		{1,0,7,8,4,2,6},
	// p 2 1
		{1,0,4,1,1,2,2},
		{0,1,9,6,2,1,1},
		{1,0,6,13,4,0,2},
	// p 2 2
		{0,2,6,4,2,0,2},
		{0,8,12,4,2,0,5},
		{1,2,4,0,1,3,6},
	// p 2 3
		{0,5,13,6,1,1,6},
		{0,5,13,6,2,1,6},
		{1,3,6,14,1,2,2},
	// p 2 4
		{0,2,12,4,2,1,5},
		{0,4,10,0,1,2,4},
		{1,3,6,0,1,2,2},

	// p 3 0
		{0,4,13,4,4,1,6},
		{0,3,7,2,1,2,2},
		{0,8,13,0,2,0,3},
	// p 3 1
		{1,1,6,3,4,2,3},
		{0,5,10,2,2,0,6},
		{0,8,12,6,1,1,1},
	// p 3 2
		{0,3,13,2,4,0,1},
		{0,8,12,0,1,1,6},
		{1,2,7,8,2,2,3},
	// p 3 3
		{0,8,12,5,1,0,5},
		{0,7,11,2,1,0,5},
		{1,0,4,2,2,2,2},
	// p 3 4
		{0,4,9,1,1,1,5},
		{0,8,13,4,1,1,5},
		{1,3,6,7,2,2,3},

	// p 4 0
		{0,4,8,6,1,1,6},
		{1,2,6,11,2,2,7},
		{1,4,6,2,1,0,5},
	// p 4 1
		{1,1,4,1,2,0,3},
		{0,1,11,6,1,1,5},
		{0,4,6,0,1,2,3},
	// p 4 2
		{1,1,6,2,4,2,2},
		{1,3,6,3,2,2,3},
		{0,5,13,6,2,1,5},
	// p 4 3
		{0,3,7,1,1,0,6},
		{0,8,13,3,2,1,5},
		{0,7,13,6,4,1,5},
	// p 4 4
		{0,1,5,6,2,1,2},
		{0,7,9,6,1,1,2},
		{0,9,13,1,2,2,3}
};

typedef struct
{
	char *current_frame, *next_frame;
	char *frames [4];
	struct sp_Rect clip;
} ENE_PANT;

ENE_PANT ene_pant [3];
struct sp_SS *sp_enemigos [4];

typedef struct
{
	char type;								// Tipo de enemigo
	unsigned char x, y;						// Posicin en pixels REAL
	unsigned char t1, t2;					// Topes.
	char sal, nu;							// Para enemigos tipo 2/3.
	char facing;							// 0 left 1 right
	char frame;
	char v;									// Velocidad para enemigos tipo 0/1
} COORDINATE;

COORDINATE ene_move[75];

/* 
	This defines where in each screen an object can be placed
	and during game "t" is the object in each screen (0 nothing 1 key 2 bolt) 
*/

typedef struct
{
	char x, y, t;
} OBJETO;

OBJETO objs [25] = {
	{6,2,0},
	{7,3,0},
	{11,3,0},
	{8,1,0},
	{1,6,0},

	{13,6,0},
	{6,4,0},
	{8,2,0},
	{13,5,0},
	{13,5,0},

	{5,4,0},
	{3,2,0},
	{13,2,0},
	{1,1,0},
	{10,2,0},

	{4,4,0},
	{12,6,0},
	{9,0,0},
	{13,1,0},
	{1,4,0},

	{13,6,0},
	{1,6,0},
	{5,6,0},
	{13,6,0},
	{1,6,0}
};

char pantalla;

/* WYZ music & sound calls */

extern char asm_number[1];
#asm
	._asm_number defb 0
#endasm

void wyz_play_sound (char fx_number)
{
	asm_number[0] = fx_number;
	#asm
		ld a, (_asm_number)
		call WYZ_PLAY_SOUND
	#endasm
}

void wyz_play_song (char song_number)
{
	play = 0;
	asm_number[0] = song_number;
	#asm
		ld a, (_asm_number)
		call WYZ_LOAD
	#endasm
	play = 1;
}

/* Auxiliary functions */

unsigned char attr (char x, char y)
{	
	char *res = (char *) (22528 + x + (y << 5));
	return (*res);
}

/* Game functions */

unsigned char move ()
{
	char falling = 0;
	char ww = (phantomas.w >> 3);		// Character positions
	char aa = (phantomas.a >> 3);
	int i; // change for this to get hyperspeed:: char i;

	if (phantomas.sal == 0)
	{
		// fall down?
		if ((phantomas.w & 7) == 0)
		{
			if ((phantomas.a & 7) == 0)
			{
				if (attr (aa, ww + 2) < 64 && attr (aa + 1, ww + 2) < 64)
				{
					phantomas.w += SPEED_FALLING;
					falling = 1;
					if (phantomas.falling == 0)
					{
						wyz_play_sound (1);
					}
					phantomas.falling = 1;
				}
				else
				{
					if (phantomas.falling)
					{
						phantomas.falling = 0;
						wyz_play_sound (6);
					}
				}
			}
			else
			{
				if (attr (aa, ww + 2) < 64 && attr (aa + 1, ww + 2) < 64 && attr (aa + 2, ww + 2) < 64)
				{
					phantomas.w += SPEED_FALLING;
					falling = 1;
					if (phantomas.falling == 0)
					{
						wyz_play_sound (1);
					}
					phantomas.falling = 1;
				}
				else
				{
					if (phantomas.falling)
					{
						phantomas.falling = 0;
						wyz_play_sound (6);
					}
				}
			}
		}
		else
		{
			phantomas.w += SPEED_FALLING;
			falling = 1;
			if (phantomas.falling == 0)
			{
				wyz_play_sound (1);
			}
			phantomas.falling = 1;
		}

		if (falling)
		{
			if (phantomas.facing)
				phantomas.next_frame = sp_phantomas_l4_a;
			else
				phantomas.next_frame = sp_phantomas_r4_a;
		}
		else
		{
			// Read keyboard to walk left/right
			
			i = sp_JoyKeyboard(&keys);

			if ((i & sp_LEFT) == 0)			// O is pressed
			{
				if (phantomas.facing == 0)
				{
					phantomas.facing = 1;
				}
				else
				{
					if ((phantomas.a & 7) == 0)
					{
						if (attr (aa - 1, ww) < 64 && attr (aa - 1, ww + 1) < 64)
						{
							phantomas.a -= SPEED_WALKING;
							phantomas.frame = (phantomas.frame + 1) & 3;
							
							if (phantomas.frame == 0)
								phantomas.next_frame = sp_phantomas_l2_a;
							else if (phantomas.frame == 1 || phantomas.frame == 3)
								phantomas.next_frame = sp_phantomas_l1_a;
							else if (phantomas.frame == 2)
								phantomas.next_frame = sp_phantomas_l3_a;					
						}
					}
					else
					{
						phantomas.a -= SPEED_WALKING;
						phantomas.frame = (phantomas.frame + 1) & 3;

						if (phantomas.frame == 0)
							phantomas.next_frame = sp_phantomas_l2_a;
						else if (phantomas.frame == 1 || phantomas.frame == 3)
							phantomas.next_frame = sp_phantomas_l1_a;
						else if (phantomas.frame == 2)
							phantomas.next_frame = sp_phantomas_l3_a;
					}
				}
			}
			else if ((i & sp_RIGHT) == 0)			// P is pressed
			{
				if (phantomas.facing == 1)
				{
					phantomas.facing = 0;
				}
				else
				{
					if ((phantomas.a & 7) == 0)
					{
						if (attr (aa + 2, ww) < 64 && attr (aa + 2, ww + 1) < 64)
						{
							phantomas.a += SPEED_WALKING;
							phantomas.facing = 0;
							phantomas.frame = (phantomas.frame + 1) & 3;

							if (phantomas.frame == 0)
								phantomas.next_frame = sp_phantomas_r2_a;
							else if (phantomas.frame == 1 || phantomas.frame == 3)
								phantomas.next_frame = sp_phantomas_r1_a;
							else if (phantomas.frame == 2)
								phantomas.next_frame = sp_phantomas_r3_a;
						}
					}
					else
					{
						phantomas.a += SPEED_WALKING;
						phantomas.facing = 0;
						phantomas.frame = (phantomas.frame + 1) & 3;

						if (phantomas.frame == 0)
							phantomas.next_frame = sp_phantomas_r2_a;
						else if (phantomas.frame == 1 || phantomas.frame == 3)
							phantomas.next_frame = sp_phantomas_r1_a;
						else if (phantomas.frame == 2)
							phantomas.next_frame = sp_phantomas_r3_a;
					}
				}
			}
			else
			{
				if (phantomas.facing)
					phantomas.next_frame = sp_phantomas_l1_a;
				else
					phantomas.next_frame = sp_phantomas_r1_a;
			}

			// Activate jumps

			if ((i & sp_UP) == 0)				// high jump
			{
				phantomas.sal = 1;
				phantomas.nu = 0;
				phantomas.mw = SPEED_JUMPY;
				phantomas.ma = SPEED_JUMPX;
				wyz_play_sound (0);
			}
			else if ((i & sp_DOWN) == 0)		// long jump
			{
				phantomas.sal = 1;
				phantomas.nu = 0;
				phantomas.mw = SPEED_JUMPX;
				phantomas.ma = SPEED_JUMPY;
				phantomas.a = (phantomas.a >> 2) << 2;
				wyz_play_sound (5);
			}
		}
	}
	else
	{
		if (phantomas.facing)
			phantomas.next_frame = sp_phantomas_l4_a;
		else
			phantomas.next_frame = sp_phantomas_r4_a;

		// Movimiento vertical

		if (phantomas.nu < HALF_NU)		// subir
		{
			if ((phantomas.w & 7) == 0)
			{
				if ((phantomas.a & 7) == 0)
				{
					if (attr (aa, ww - 1) < 64 && attr (aa + 1, ww - 1) < 64)
						phantomas.w -= phantomas.mw;
				}
				else
				{
					if (attr (aa, ww - 1) < 64 && attr (aa + 1, ww - 1) < 64 && attr (aa + 2, ww - 1) < 64)
						phantomas.w -= phantomas.mw;
				}
			}
			else
				phantomas.w -= phantomas.mw;
		}
		else if (phantomas.nu < MAX_NU)	// bajar
		{
			if ((phantomas.w & 7) == 0)
			{
				if ((phantomas.a & 7) == 0)
				{
					if (attr (aa, ww + 2) < 64 && attr (aa + 1, ww + 2) < 64)
						phantomas.w += phantomas.mw;
					else 
						if (phantomas.sal == 1)
						{	
							phantomas.sal = 0;
							wyz_play_sound (6);
						}	
				}
				else
				{
					if (attr (aa, ww + 2) < 64 && attr (aa + 1, ww + 2) < 64 && attr (aa + 2, ww + 2) < 64)
						phantomas.w += phantomas.mw;
					else 
						if (phantomas.sal == 1)
						{
							phantomas.sal = 0;
							wyz_play_sound (6);
						}	
				}
			}
			else
				phantomas.w += phantomas.mw;
		}
		else
		{
			phantomas.sal = 0;
		}

		ww = phantomas.w >> 3;

		// Movimiento horizontal
		// hay que comprobar de pixel en pixel pues la pos. puede ser impar.

		if (phantomas.facing)				// izquierda
		{
			if ((phantomas.a & 7) == 0)
			{
				if ((phantomas.w & 7) == 0)
				{
					if (attr (aa - 1, ww) < 64 && attr (aa - 1, ww + 1) < 64)
					phantomas.a -= phantomas.ma;
				}
				else
				{
					if (attr (aa - 1, ww) < 64 && attr (aa - 1, ww + 1) < 64 && attr (aa - 1, ww + 2) < 64)
						phantomas.a -= phantomas.ma;
				}
			}
			else
				phantomas.a -= phantomas.ma;
		}
		else								// derecha
		{
			if ((phantomas.a & 7) == 0)
			{
				if ((phantomas.w & 7) == 0)
				{
					if (attr (aa + 2, ww) < 64 && attr (aa + 2, ww + 1) < 64)
						phantomas.a += phantomas.ma;
				}
				else
				{
					if (attr (aa + 2, ww) < 64 && attr (aa + 2, ww + 1) < 64 && attr (aa + 2, ww + 2) < 64)
						phantomas.a += phantomas.ma;
				}
			}
			else
				phantomas.a += phantomas.ma;
		}	

		phantomas.nu ++;

		if (phantomas.sal == 0)
		{
			// ajusto a coordenadas pares antes de terminar:

			phantomas.a = (phantomas.a >> 1) << 1;
			phantomas.w = (phantomas.w >> 2) << 2;

			if ((phantomas.a & 7) == 0)
			{
				if (attr (aa, ww + 2) > 64 || attr (aa + 1, ww + 2) > 64)
					wyz_play_sound (6);
						
			}
			else
			{
				if (attr (aa, ww + 2) > 64 || attr (aa + 1, ww + 2) > 64 || attr (aa + 2, ww + 2) > 64)
					wyz_play_sound (6);
			}
		}
	}
}

/* This function is called to tint an already created sprite: */

uchar whatColour;
void tintSprite (struct sp_CS *cs)
{
	cs->colour = whatColour;
}

/* Initialize enemies */

void initialize_ene_move ()
{
	char i;

	for (i = 0; i < 75; i ++)
	{
		ene_move [i].type = ene_mapa [i].type;

		if (ene_mapa [i].type == 0)
		{
			ene_move [i].x = 8 + 16 * ene_mapa [i].t1;
			ene_move [i].y = 48 + 16 * ene_mapa [i].p;
			ene_move [i].t1 = ene_move [i].x;
			ene_move [i].t2 = 8 + 16 * ene_mapa [i].t2;
		}
		else
		{
			ene_move [i].x = 8 + 16 * ene_mapa [i].p;
			ene_move [i].y = 48 + 16 * ene_mapa [i].t1;	
			ene_move [i].t1 = ene_move [i].y;
			ene_move [i].t2 = 48 + 16 * ene_mapa [i].t2;
		}

		ene_move [i].v = ene_mapa [i].v;
		ene_move [i].facing = 1;
		ene_move [i].frame = 0;
	}
}

void init_enem ()
{
	char i, idx;

	for (i = 0; i < 3; i ++)
	{
		idx = 3 * pantalla + i;

		if (ene_move [idx].type == 0 || ene_move [idx].type == 1)
		{
			if (ene_move [idx].type == 0)             // Horizontal
			{
				// Clipping region:

				ene_pant [i].clip.row_coord = 6 + 2 * ene_mapa [idx].p;
				ene_pant [i].clip.col_coord = 1 + 2 * ene_mapa [idx].t1;
				ene_pant [i].clip.width = 2 + 2 * ene_mapa [idx].t2 - 2 * ene_mapa[idx].t1;
				ene_pant [i].clip.height = 2;
			}
			else									// Vertical
			{
				ene_pant [i].clip.row_coord = 6 + 2 * ene_mapa [idx].t1;
				ene_pant [i].clip.col_coord = 1 + 2 * ene_mapa [idx].p;
				ene_pant [i].clip.width = 2;
				ene_pant [i].clip.height = 2 + 2 * ene_mapa [idx].t2 - 2 * ene_mapa[idx].t1;
			}
		
		}
		/*
		else
		{
			// TODO: los otros tipos de enemigos (no salen en infinity)
		}
		*/

		/* Asignamos los frames segn el campo spr: */

		switch (ene_mapa [idx].spr)
		{
			case 0: // Patrol

				ene_pant [i].frames [0] = sp_patrol_l1_a;
				ene_pant [i].frames [1] = sp_patrol_l2_a;
				ene_pant [i].frames [2] = sp_patrol_r1_a;
				ene_pant [i].frames [3] = sp_patrol_r2_a;

				break;
			
			case 1:	// Mr. Roboto (domo arigato)

				ene_pant [i].frames [0] = sp_roboto_r1_a;
				ene_pant [i].frames [1] = sp_roboto_r2_a;
				ene_pant [i].frames [2] = sp_roboto_r1_a;
				ene_pant [i].frames [3] = sp_roboto_r2_a;			

				break;

			case 2:	// Hyapso (where's Ken-Chan?)

				ene_pant [i].frames [0] = sp_hyapso_r1_a;
				ene_pant [i].frames [1] = sp_hyapso_r2_a;
				ene_pant [i].frames [2] = sp_hyapso_r1_a;
				ene_pant [i].frames [3] = sp_hyapso_r2_a;			
					
			// etctera.
		}

		// Reiniciando la animacin:

		ene_pant [i].next_frame = ene_pant [i].frames [0];

		// Tint sprite:

		whatColour = ene_mapa [idx].c;
		sp_IterateSprChar (sp_enemigos [i], tintSprite);

		// Thanks Alvin, you explained this to me :)
	}
}

void move_enem ()
{
	char i, idx;

	for (i = 0; i < 3; i ++)
	{
		idx = 3 * pantalla + i;

		if (ene_move [idx].type < 2)
		{
			if (ene_move [idx].type == 0)
				ene_move [idx].x += ene_move [idx].v;
			else
				ene_move [idx].y += ene_move [idx].v;

			if (
					(!ene_move[idx].type && (ene_move [idx].x <= ene_move [idx].t1 || ene_move [idx].x >= ene_move [idx].t2))
				||
					(ene_move[idx].type && (ene_move [idx].y <= ene_move [idx].t1 || ene_move [idx].y >= ene_move [idx].t2))
			   )
			{
				ene_move [idx].v = -ene_move [idx].v;
				if (ene_move [idx].v < 0)
					ene_move [idx].facing = 0;
				else
					ene_move [idx].facing = 1;
			}	
		}

		/* TODO en futuros juegos: case 3 y case 2 (Moonage Daydream) */

		ene_move [idx].frame = 1 - ene_move [idx].frame;
		
		if (ene_move [idx].facing == 0)
			ene_pant [i].next_frame = ene_pant [i].frames [ene_move [idx].frame];
		else
			ene_pant [i].next_frame = ene_pant [i].frames [2 + ene_move [idx].frame];
	}
}

void init_phantomas ()
{
	phantomas.w = 48;
	phantomas.a = 24;
	phantomas.sal = 0;
	phantomas.nu = 0;
	phantomas.frame = 0;
	phantomas.facing = 0;
	phantomas.life = LIFE;
	phantomas.keys = 0;
	phantomas.bolts = 0;
	phantomas.time_left = TIME;
	phantomas.time_counter = COUNTER_SET;
}

// This sets the tileset

void set_tileset ()
{
	uchar i; char *pointer = game_tiles;

	for (i = 32; i < 136; i++)
	{
		sp_TileArray (i, pointer);
		pointer+=8;
	}
}

// This sets the atari font

/*void set_atari ()
{
	char i;

	for (i = 32; i < 97; i++)
		sp_TileArray (i, font_atari [i-32]);
}*/

void draw_screen ()
{
	char x = 1, y = 6;
	unsigned char c;
	unsigned char ch;
	int idx = pantalla * 120;
	int idx_attr;
	uchar i;

	sp_ClearRect(spritesClip, INK_GREEN | PAPER_BLACK, ' ', sp_CR_TILES); 

	for (i = 0; i < 120; i ++)
	{
		c = map [idx + i];

		if (c > 0)
		{
			idx_attr = c << 2;

			ch = attrs[idx_attr];
			sp_PrintAt (y, x, ch, 32 + idx_attr);
			idx_attr ++;
			ch = attrs[idx_attr];
			sp_PrintAt (y, x + 1, ch, 32 + idx_attr);
			idx_attr ++;
			ch = attrs[idx_attr];
			sp_PrintAt (y + 1, x, ch, 32 + idx_attr);
			idx_attr ++;
			ch = attrs[idx_attr];
			sp_PrintAt (y + 1, x + 1, ch, 32 + idx_attr);
		}
		else
			if ((rand()&3) == 0)
				sp_PrintAt (y + (rand() & 1), x, 4, 43);

		x += 2; if (x == 31) { x = 1; y += 2; }
	}

	sp_Invalidate(spritesClip, spritesClip);
}

void spread_objects ()
{
	char i;
	char p;

	// Empty all first:

	for (i = 0; i < 25; i ++)
		objs[i].t = 0;

	// Spread keys & bolts:

	for (i = 0; i < 10; i ++)
	{
		do
		{
			p = rand () % 25;
		} while (objs [p].t);

		objs [p].t = 1;

		do
		{
			p = rand () % 25;
		} while (objs [p].t);

		objs [p].t = 2;
	}
}

void draw_tile (char x, char y, char c, char tile)
{
	sp_PrintAtInv (6 + y, 1 + x, c, tile);
	sp_PrintAtInv (6 + y, 2 + x, c, tile + 1);
	sp_PrintAtInv (7 + y, 1 + x, c, tile + 2);
	sp_PrintAtInv (7 + y, 2 + x, c, tile + 3);
}

void draw_objects()
{
	if (objs [pantalla].t == 1)
	{
		draw_tile (objs[pantalla].x << 1, objs[pantalla].y << 1, 6, 128);
	}
	else if (objs [pantalla].t == 2)
	{
		draw_tile (objs[pantalla].x << 1, objs[pantalla].y << 1, 5, 132);
	}
}

void draw_life ()
{
	sp_PrintAtInv (4, 5, 71, 33 + phantomas.life / 100);
	sp_PrintAtInv (4, 6, 71, 33 + (phantomas.life % 100) / 10);
	sp_PrintAtInv (4, 7, 71, 33 + phantomas.life % 10);
}

void draw_keys ()
{
	sp_PrintAtInv (4, 17, 71, 33 + phantomas.keys / 10);
	sp_PrintAtInv (4, 18, 71, 33 + phantomas.keys % 10);
}

void draw_bolts ()
{
	sp_PrintAtInv (4, 28, 71, 33 + phantomas.bolts / 10);
	sp_PrintAtInv (4, 29, 71, 33 + phantomas.bolts % 10);
}

void draw_time ()
{
	sp_PrintAtInv (4, 27, 66, 33 + phantomas.time_left / 100);
	sp_PrintAtInv (4, 28, 66, 33 + (phantomas.time_left % 100) / 10);
	sp_PrintAtInv (4, 29, 66, 33 + phantomas.time_left % 10);
}

void redraw_tile ()
{
	int x, y;
	uchar c;
	uchar idx_attr;
	uchar ch;

	x = objs [pantalla].x;
	y = objs [pantalla].y;
	c = map [pantalla * 120 + x + 15 * y];

	x = 1 + (x << 1); y= 6 + (y << 1);	// adjust to screen

	if (c > 0)
	{
		idx_attr = c << 2;

		ch = attrs[idx_attr];
		sp_PrintAtInv (y, x, ch, 32 + idx_attr);
		ch = attrs[idx_attr+1];
		sp_PrintAtInv (y, x + 1, ch, 32 + idx_attr+1);
		ch = attrs[idx_attr+2];
		sp_PrintAtInv (y + 1, x, ch, 32 + idx_attr+2);
		ch = attrs[idx_attr+3];
		sp_PrintAtInv (y + 1, x + 1, ch, 32 + idx_attr+3);
	}
	else
	{
		sp_PrintAtInv (y, x, 4, 32);
		sp_PrintAtInv (y, x + 1, 4, 32);
		sp_PrintAtInv (y + 1, x, 4, 32);
		sp_PrintAtInv (y + 1, x + 1, 4, 32);
	}


}

/*void stop_ay (void)
{
	#asm
		ld a, (WYZ_INTERR)
		res 1, a
		ld (WYZ_INTERR), a
		ld a, 0
		ld ($62D3), a
		ld ($62D4), a
		ld ($62D5), a
	#endasm
}*/

void muerte (uchar a, uchar b)
{
	char i, x, y;
	char *screen_attrs_add;
	
//	stop_ay ();

	draw_life ();
	sp_UpdateNow ();
	wyz_play_sound (3);
	sp_Wait (20);

	for (i=0; i < 20; i++)
	{
		for (y = 6; y < 22; y++)
			for (x = 1; x < 31; x ++)
			{
				screen_attrs_add = (char *) (22528 + (y<<5) + x);
				if (i & 1)
					*screen_attrs_add = a;
				else
					*screen_attrs_add = b;
			}
		wyz_play_sound (4);

	}

	wyz_play_sound (1);
	sp_ClearRect(spritesClip, INK_GREEN | PAPER_BLACK, ' ', sp_CR_TILES); 
	sp_PrintAtInv (4, 27, 0, 32);
	sp_UpdateNow();
}

/* Uncrunch image routine */

void uncrunch (int address)
{
	unsigned char *origin;
	unsigned char *destiny;
	unsigned char msb, lsb;

	origin = (unsigned char *) PUCRUNCH;
	destiny = (unsigned char *) PUCRUNCH + 2;

	msb = address >> 8;
	lsb = address & 255;
	
	*origin = lsb;
	origin ++;
	*origin = msb;

	*destiny = 0;
	destiny ++;
	*destiny = 64;

	// Antes de llamar a pucrunch, tintamos la pantalla de negro
	// para que no se vea tan feo:

	origin = (unsigned char *) 22528;
	while (origin < 23296)
	{
		*origin = 0;
		origin ++;
	}

	#asm
		call PUCRUNCH + 4
	#endasm
}

/* Main */

void main (void)
{
	unsigned char j, x, y;
	int i;	// Cambiar esto por char i; para que el juego vaya follao de rpido :D
	char idx;
	struct sp_SS *sp_phantomas;
	char flag1, flag2;
	char *timer;
	unsigned char *wyz_music_flag;

	wyz_music_flag = 25263;

	// Ugly pseudorandom seed using the FRAMES sys var
	
	timer = 23672;
	srand (*timer);

	/* Substitute BASICs ISR for this thingo: */

	/* Add music handler here in the ISR! */

	#asm
		di
	#endasm
	
	sp_InitIM2(0xf1f1);				// Does nothing :)
	sp_CreateGenericISR(0xf1f1); 
	sp_RegisterHook(255, musicISR);  // Registramos el hook - Gracias Utopian!
	
	#asm 
		ei 
	#endasm 

	/* Initialization stuff */
	
	sp_Initialize (INK_WHITE | PAPER_BLACK, ' ');
	sp_Border (BLACK);
	sp_AddMemory(0, 42, 14, AD_FREE);		// Allocate for 42 blocks of memory (tight!)
	ps.flags  = sp_PSS_INVALIDATE;
    ps.bounds = sp_ClipStruct;

	/* Clip structure -- define it as small as possible! */

	spritesClipValues.row_coord = 6;
	spritesClipValues.col_coord = 1;
	spritesClipValues.height = 16;
	spritesClipValues.width = 30;
	spritesClip = &spritesClipValues;

	/* Create sprites */

	sp_phantomas = sp_CreateSpr(sp_OR_SPRITE, 3, sp_phantomas_r4_a, 1, TRANSPARENT);
	sp_AddColSpr (sp_phantomas,  sp_phantomas_r4_b, TRANSPARENT);
	sp_AddColSpr (sp_phantomas, sp_phantomas_r4_c, TRANSPARENT);

	sp_MoveSprAbs(sp_phantomas, spritesClip, 0, phantomas.w >> 3, phantomas.a >> 3, phantomas.a & 7, phantomas.w & 7);

	for (i = 0; i < 3; i++)
	{
		sp_enemigos [i] = sp_CreateSpr(sp_OR_SPRITE, 3, sp_patrol_l1_a, 1, 2 + 2 * i);
		sp_AddColSpr (sp_enemigos [i], sp_patrol_l1_b, 2 + 2 * i);
		sp_AddColSpr (sp_enemigos [i], sp_patrol_l1_c, 2 + 2 * i);

		sp_MoveSprAbs (sp_enemigos [i], &ene_pant [i].clip, 0, ene_move [i].y >> 3, ene_move [i].x >> 3, ene_move [i].x & 7, ene_move [i].y & 7);
		ene_pant [i].current_frame = ene_pant [i].next_frame = sp_patrol_l1_a;
	}

	phantomas.current_frame = phantomas.next_frame = sp_phantomas_r4_a;	

	/* Create tiles */

	// set_atari();
	
	sp_ClearRect(sp_ClipStruct, 0, ' ', sp_CR_TILES); 
	
	sp_Invalidate(sp_ClipStruct, sp_ClipStruct);
	sp_UpdateNow();

	// Recuerda cambiar esto por una llamada a pucrunch

	uncrunch (SCR_CEZ);	// logo CEZ

	wyz_play_sound (3);
	play = 1;
	
	sp_Pause (100);

quefeo:

	while (1)
	{
		wyz_play_song (0);

		sp_ClearRect(sp_ClipStruct, 0, ' ', sp_CR_TILES); 
		sp_UpdateNow();

		uncrunch (SCR_MARCADOR);

		sp_UpdateNow ();

		sp_Pause (32767);

		play = 0;

		/* Keyboard definition */

		keys.up    = sp_LookupKey('q');
		keys.down  = sp_LookupKey('a');
		keys.left  = sp_LookupKey('o');
		keys.right = sp_LookupKey('p');
		keys.fire  = sp_LookupKey(' ');

		/* Para msica, toca sonido 3, espera, pone msica */

//		stop_ay ();

		wyz_play_sound(3);

/*		for (i=0; i<5000; i++)	// Bucle tontorrn para perder tiempo, super feo
		{
			play = 1;
		}*/

		wyz_play_song (1);

		/* INIT */

		init_phantomas ();
		initialize_ene_move ();
		pantalla = 20;
		flag1 = 0;
		set_tileset ();
		draw_screen ();
		draw_life ();
		draw_keys ();
		draw_bolts ();
		spread_objects ();

		while (!flag1)
		{
			draw_screen ();
			init_enem ();
			draw_objects ();

			// Pongo los sprites en su sitio:

			idx = pantalla * 3;
			for (i = 0; i < 3; i++)
			{
				sp_MoveSprAbs (sp_enemigos [i], &ene_pant [i].clip, ene_pant [i].next_frame - ene_pant [i].current_frame, ene_move [idx].y >> 3, ene_move [idx].x >> 3, ene_move [idx].x & 7, ene_move [idx].y & 7);
				ene_pant [i].current_frame = ene_pant [i].next_frame;
				idx ++;
			}

			sp_MoveSprAbs(sp_phantomas, spritesClip, phantomas.next_frame - phantomas.current_frame, phantomas.w >> 3, phantomas.a >> 3, phantomas.a & 7, phantomas.w & 7);
			phantomas.current_frame = phantomas.next_frame;

			// Bandera de salida de la pantalla:
		
			flag2 = 0;

			// Bucle pantallstico.
			
			while (!flag2)
			{
				sp_UpdateNow();
				
				/* Move sprites */

				move ();
				move_enem ();

				/* Collisions */

				// 1.- With enemies

				idx = pantalla * 3;
				for (i = 0; i < 3; i ++)
				{
					if (ene_move [idx].x >= phantomas.a - 12 && ene_move [idx].x <= phantomas.a + 12 && ene_move [idx].y >= phantomas.w - 12 && ene_move [idx].y <= phantomas.w + 12)
						if (phantomas.life > 0)
						{
							phantomas.life --;
							draw_life ();
							if (phantomas.life & 1)
								wyz_play_sound (4);

							if (phantomas.life == 0)
							{
								muerte (56, 7);

								flag2 = 1;
								flag1 = 1;
							}
						}
					idx ++;
				}

				// 2.- With keys / bolts

				if (objs [pantalla].t != 0)
				{
					if ( ((phantomas.a - 8) >> 4) == objs [pantalla].x && ((phantomas.w - 48) >> 4) == objs [pantalla].y)
					{
						j = 0;
						if (objs [pantalla].t == 1)
						{
							phantomas.keys ++;
							wyz_play_sound (2);
							redraw_tile ();
							objs [pantalla].t = 0;
							j ++;
						}
						else if (objs [pantalla].t == 2 && phantomas.keys > 0)
						{
							phantomas.keys --;
							phantomas.bolts ++;
							wyz_play_sound (2);
							draw_bolts ();
							redraw_tile ();
							objs [pantalla].t = 0;

							if (phantomas.bolts == 10)
								draw_time ();

							j ++;
						}

						if (j)
						{
							if (phantomas.life < (LIFE - ADDLIFE))
								phantomas.life += ADDLIFE;
							else
								phantomas.life = LIFE;

							draw_keys ();
							draw_life ();
						}
					}
				}

				// Countdown!

				if (phantomas.bolts == 10)
				{
					phantomas.time_counter --;

					if (phantomas.time_counter == 0)
					{
						phantomas.time_counter = COUNTER_SET;
						phantomas.time_left --;

						draw_time ();
						wyz_play_sound (3);

						if (phantomas.time_left == 0)
						{
							if (pantalla % 5 != 0)
							{
								muerte (22, 50);
								
								flag2 = 1;
								flag1 = 1;
							}
							else
							{
								// Success!!

								// Insert here FIN
								
								// Recuerda cambiar esto por una llamada a pucrunch

								uncrunch (SCR_FIN);

								// a shupahla.
								
								sp_Pause (32767);

								flag2 = 1;
								flag1 = 1;

								goto quefeo;
							}
						}
					}
				}

				/* Render */

				idx = pantalla * 3;
				for (i = 0; i < 3; i++)
				{
					sp_MoveSprAbs (sp_enemigos [i], &ene_pant [i].clip, ene_pant [i].next_frame - ene_pant [i].current_frame, ene_move [idx].y >> 3, ene_move [idx].x >> 3, ene_move [idx].x & 7, ene_move [idx].y & 7);
					ene_pant [i].current_frame = ene_pant [i].next_frame;
					idx ++;
				}

				sp_MoveSprAbs(sp_phantomas, spritesClip, phantomas.next_frame - phantomas.current_frame, phantomas.w >> 3, phantomas.a >> 3, phantomas.a & 7, phantomas.w & 7);
				phantomas.current_frame = phantomas.next_frame;

				/* Connectivity */

				if (phantomas.a <= 0)
				{
					phantomas.a = 236;
					pantalla --;
					flag2 = 1;
				}
				else if (phantomas.a >= 240)
				{
					phantomas.a = 4;
					pantalla ++;
					flag2 = 1;
				}
				else if (phantomas.w <= 40 && pantalla > 4)
				{
					phantomas.w = 152;
					pantalla -= 5;
					flag2 = 1;
				}
				else if (phantomas.w >= 168)
				{
					phantomas.w = 44;
					pantalla += 5;
					flag2 = 1;
				}
			}
		}
	}
}
